﻿"""
積算の結果を使用してグラフを出力します。
詳細は README.pdf を参照してください。

Copyright (C) 2024 UNION SYSTEM Inc.

"""

import os
import pandas as pd
import readcsv as rcsv
import ss7job
import plot_graph as pg


# コンクリート
def conc_job(
    df_rc_dat: pd.DataFrame,
    df_con_dat: pd.DataFrame,
    str_res: str,
) -> pd.DataFrame:
    """
    コンクリートのデータフレームを作成

    Args:
        df_rc_dat (pd.DataFrame): RCデータのデータフレーム
        df_con_dat (pd.DataFrame): コンクリートのデータフレーム
        str_res (str): 結果セット名

    Returns:
        pd.DataFrame: コンクリートのデータフレーム
    """
    # コンクリート
    bln_flag: bool = False
    for int_index in range(len(df_rc_dat)):
        if df_rc_dat.loc[int_index, "Header1"] == "コンクリート体積[m3]":
            bln_flag = True
        if bln_flag:
            if df_rc_dat.loc[int_index, "Header2"] == "合計":
                lst_rc_dat: list = []
                for str_rc_item in pg.RC_ITEM:  # type str
                    lst_rc_dat.append(df_rc_dat.loc[int_index, str_rc_item])
                if len(df_con_dat) == 0:
                    df_con_dat = pd.DataFrame(
                        str_to_float(lst_rc_dat),
                        columns=[str_res],
                        index=df_rc_dat.columns[3:14],
                    )
                else:
                    df_con_dat[str_res] = str_to_float(lst_rc_dat)
                break
    return df_con_dat


# 型枠
def form_job(
    df_rc_dat: pd.DataFrame,
    df_kata_dat: pd.DataFrame,
    str_res: str,
) -> pd.DataFrame:
    """
    _summary_

    Args:
        df_rc_dat (pd.DataFrame): RCデータのデータフレーム
        df_kata_dat (pd.DataFrame): 型枠のデータフレーム
        str_res (str): 結果セット名

    Returns:
        pd.DataFrame: _description_
    """
    for int_index in range(len(df_rc_dat)):
        if df_rc_dat.loc[int_index, "Header1"] == "　　型枠面積　　[m2]":
            lst_rc_dat: list = []
            for str_rc_item in pg.RC_ITEM:  # type str
                lst_rc_dat.append(df_rc_dat.loc[int_index, str_rc_item])
            if len(df_kata_dat) == 0:
                df_kata_dat = pd.DataFrame(
                    str_to_float(lst_rc_dat),
                    columns=[str_res],
                    index=df_rc_dat.columns[3:14],
                )
            else:
                df_kata_dat[str_res] = str_to_float(lst_rc_dat)
    return df_kata_dat


# 鉄筋
def rein_job(
    df_rc_dat: pd.DataFrame,
    df_rein_dat: pd.DataFrame,
    str_res: str,
) -> pd.DataFrame:
    """
    _summary_

    Args:
        df_rc_dat (pd.DataFrame): RCデータのデータフレーム
        df_rein_dat (pd.DataFrame): 鉄筋のデータフレーム
        str_res (str): 結果セット名

    Returns:
        pd.DataFrame: _description_
    """
    bln_flag: bool = False
    for int_index in range(len(df_rc_dat)):
        if df_rc_dat.loc[int_index, "Header1"] == "　　鉄筋重量　　 [t]":
            bln_flag = True
        if bln_flag:
            if df_rc_dat.loc[int_index, "Header2"] == "合計":
                lst_rc_dat: list = []
                for str_rc_item in pg.RC_ITEM:  # type str
                    lst_rc_dat.append(df_rc_dat.loc[int_index, str_rc_item])
                if len(df_rein_dat) == 0:
                    df_rein_dat = pd.DataFrame(
                        str_to_float(lst_rc_dat),
                        columns=[str_res],
                        index=df_rc_dat.columns[3:14],
                    )
                else:
                    df_rein_dat[str_res] = str_to_float(lst_rc_dat)
                break
    return df_rein_dat


# コンクリート，型枠，鉄筋の処理
def rc_job(
    str_csv_path_rc: str,
    str_res: str,
    df_con_dat: pd.DataFrame,
    df_kata_dat: pd.DataFrame,
    df_rein_dat: pd.DataFrame,
) -> pd.DataFrame:
    """
    結果CSVファイルから、コンクリート、型枠、鉄筋データを取得し、
    表示するためのデータフォーマットに整えます。

    Args:
        str_csv_path_rc (str): CSVファイルのパス
        str_res (str): 結果名
        df_con_dat (pd.DataFrame): コンクリートデータ
        df_kata_dat (pd.DataFrame): 型枠データ
        df_rein_dat (pd.DataFrame): 鉄筋データ

    Returns:
        pd.DataFrame: 表示のためのコンクリート、型枠、鉄筋データ
    """
    # 結果CSVの「部位別集計表(コンクリート，型枠，鉄筋)」をpandasにセットする
    lst_data_rc: list = rcsv.get_data(
        str_csv_path_rc, "部位別集計表(コンクリート，型枠，鉄筋)"
    )
    if lst_data_rc is not None:
        # columnsに登録する名称を変種する
        lst_data_rc[1][0] = "Header1"
        lst_data_rc[1][1] = "Header2"
        lst_data_rc[1][2] = "Header3"
        lst_name_rc: list = lst_data_rc[1]
        del lst_data_rc[0:3]
        df_rc_dat = pd.DataFrame(lst_data_rc, columns=lst_name_rc)
        # コンクリート
        df_con_dat = conc_job(df_rc_dat, df_con_dat, str_res)

        # 型枠
        df_kata_dat = form_job(df_rc_dat, df_kata_dat, str_res)

        # 鉄筋
        df_rein_dat = rein_job(df_rc_dat, df_rein_dat, str_res)

    return df_con_dat, df_kata_dat, df_rein_dat


# 鉄骨の処理
def s_job(
    str_csv_path_s: str, str_res: str, df_steel_dat: pd.DataFrame
) -> pd.DataFrame:
    """
    結果CSVファイルから、鉄骨データを取得し、
    表示するためのデータフォーマットに整えます。

    Args:
        str_csv_path_s (str): _description_
        str_res (str): _description_
        dfSteelDat (pd.DataFrame): _description_

    Returns:
        pd.DataFrame: 表示のための鉄骨データ
    """
    # 結果CSVの「部位別集計表(鉄骨)」をpandasにセットする
    lst_data_s: list = rcsv.get_data(str_csv_path_s, "部位別集計表(鉄骨)")
    if lst_data_s is not None:
        # columnsに登録する名称を変種する
        lst_data_s[1][0] = "Header1"
        lst_data_s[1][1] = "Header2"
        lst_data_s[1][2] = "Header3"
        lst_data_s[1][3] = "Header4"
        lst_data_s[1][4] = "Header5"
        lst_name_s: list = lst_data_s[1]
        del lst_data_s[0:3]
        df_s_dat = pd.DataFrame(lst_data_s, columns=lst_name_s)

        # スライスインデクスについて
        #   5:11 ... "柱":"その他"
        # 鉄骨
        # "Header1"が"　　　 合計　　[t]"となる行を探す。
        for int_index in range(len(df_s_dat)):
            if df_s_dat.loc[int_index, "Header1"] == "　　　 合計　　[t]":
                lst_s_dat: list = []
                for str_s_item in pg.S_ITEM:  # type str
                    lst_s_dat.append(df_s_dat.loc[int_index, str_s_item])
                if len(df_steel_dat) == 0:
                    df_steel_dat = pd.DataFrame(
                        str_to_float(lst_s_dat),
                        columns=[str_res],
                        index=df_s_dat.columns[5:11],
                    )
                else:
                    df_steel_dat[str_res] = str_to_float(lst_s_dat)
    return df_steel_dat


# 文字列の実数変換
def str_to_float(lst_val: list) -> list:
    """
    文字列を実数に変換します。
    空白となっている箇所は0.0をセットします。

    Args:
        lst_val (list): 文字列の値

    Returns:
        list: 実数の値
    """
    lst_dat: list = []
    for val in lst_val:
        if val == "":
            lst_dat.append(0.0)
        else:
            lst_dat.append(float(val))
    return lst_dat


# 実行関数
def cost_graph(str_path: str, lst_res: list) -> None:
    """
    実行関数

    Args:
        str_path (str): 物件フォルダのパス
        lst_res (list): 結果セット名リスト
    """

    df_con_dat = pd.DataFrame()  # コンクリートデータ
    df_kata_dat = pd.DataFrame()  # 型枠データ
    df_rein_dat = pd.DataFrame()  # 鉄筋データ
    df_steel_dat = pd.DataFrame()  # 鉄骨データ

    # 部位別集計表の結果CSVのパス
    str_csv_path_rc: str = str_path + "\\積算RC.csv"  # コンクリート，型枠，鉄筋
    str_csv_path_s: str = str_path + "\\積算S.csv"  # 鉄骨

    # 指定した結果の数だけ、データを取得する
    for str_res in lst_res:

        # コンクリート，型枠，鉄筋について
        if ss7job.make_cost_result_csv(str_path, str_csv_path_rc, str_res, 1):
            if os.path.exists(str_csv_path_rc):
                df_con_dat, df_kata_dat, df_rein_dat = rc_job(
                    str_csv_path_rc, str_res, df_con_dat, df_kata_dat, df_rein_dat
                )

        # 鉄骨について
        if ss7job.make_cost_result_csv(str_path, str_csv_path_s, str_res, 2):
            if os.path.exists(str_csv_path_s):
                df_steel_dat = s_job(str_csv_path_s, str_res, df_steel_dat)

    # グラフの表示
    pg.plot_graph(df_con_dat, df_kata_dat, df_rein_dat, df_steel_dat, lst_res)
